/*
 * File: ChaosGame.java
 * ================================================================
 * An implementation of the Chaos Game.  The program picks three
 * points to act as vertices of a triangle (these will be the
 * upper-left, lower-right, and lower-left corners of the window).
 * Starting with the top point, we repeatedly do the following:
 * 
 * 1. Choose a random one of the three corners.
 * 2. Move halfway between our current location and that corner.
 * 3. Put a dot at our location.
 * 
 * The result is pretty surprising!
 */
import acm.graphics.*;
import acm.program.*;
import acm.util.*;
import java.awt.*;

public class ChaosGame extends GraphicsProgram {	
	public void run() {
		/* Start at (0, 0), one of the vertices. */
		GPoint pt = new GPoint(0, 0);
		
		/* Keep moving randomly and plotting pixels. */
		while (true) {
			moveRandomly(pt);
			plotPixel(pt);
		}
	}
	
	/**
	 * Draws a pixel at the indicated point.
	 * 
	 * @param pt The point where the pixel should be drawn.
	 */
	private void plotPixel(GPoint pt) {
		GRect pixel = new GRect(pt.getX(), pt.getY(), 1, 1);
		pixel.setFilled(true);
		add(pixel);
	}
	
	/**
	 * Chooses and returns a random one of three corners of the invisible
	 * triangle, which are the corners of the window except for the bottom-
	 * right corner.
	 * 
	 * @return A random corner of the world.
	 */
	private GPoint chooseRandomCorner() {
		/* Choose which corner to return. */
		RandomGenerator rgen = RandomGenerator.getInstance();
		int corner = rgen.nextInt(1, 3);
		
		/* Based on which corner is selected, return the appropriate point. */
		if (corner == 1) {
			return new GPoint(0, 0);
		} else if (corner == 2) {
			return new GPoint(getWidth(), 0);
		} else {
			return new GPoint(0, getHeight());
		}
	}
	
	/**
	 * Moves the specified point halfway toward a randomly-chosen corner.
	 * 
	 * @param pt The point to move.
	 */
	private void moveRandomly(GPoint pt) {
		GPoint d = chooseRandomCorner();
		
		pt.setLocation((pt.getX() + d.getX()) / 2.0,
				       (pt.getY() + d.getY()) / 2.0);
	}
}




